class SearchController < ApplicationController
  
  def index

    @types = LearningObject.object_types

    @topics = Topic.defaults

    @school_levels = ['Educação Infantil', 'Ensino Fundamental', 'Ensino Médio'].map { |o| Topic.find_by_name(o) }

    @source = Institution.all
  end

  def fetch
    params[:query] = "*" if params[:query].blank?

    case params[:search_class]
    when "LearningObject"
      @result = LearningObject.search params[:query], where: where_hash(params), order: lo_order_hash(params), page: params[:page] || 1, per_page: 10
    when "Collection"
      @result = Collection.search params[:query], where: {privacy: "public"}, order: col_order_hash(params), page: params[:page]
    when "User"
      @result = User.search params[:query], order: user_order_hash(params), page: params[:page]
    else 
      raise "Wrong search class parameter"
    end

    render partial: 'search/fetch'
  end

  def autocomplete
    case params[:search_class]
    when "LearningObject"
      render json: LearningObject.search(params[:query], {
        fields: ['name^5', 'description', 'author'],
        limit: 10,
        load: false,
        misspellings: { below: 5 }
      }).map(&:name)
    when "Collection"
      render json: Collection.search(params[:query], {
        where: {privacy: "public"},
        fields: ['name^5', 'description', 'owner'],
        limit: 10,
        load: false,
        misspellings: { below: 5 }
      }).map(&:name)
    when "User"
      render json: User.search(params[:query], {
        fields: ['name'],
        limit: 10,
        load: false,
        misspellings: { below: 5 }
      }).map(&:name)
    else 
      raise "Wrong search class parameter"
    end
  end

  private

  def where_hash(params)
    hash = {}

    topics = params[:subject] unless params[:subject].blank?
    topics += ", " unless topics.nil?
    topics = topics.to_s + params[:school_level] unless params[:school_level].blank?

    # build hash
    hash[:topics_name] = topics.split(', ') unless topics.nil?
    hash[:object_type] = params[:type].split(', ') unless params[:type].blank?
    # year = params[:year].blank? ? nil : params[:year].split('-').take(2)

    hash.blank? ? nil : hash
  end

  def lo_order_hash(params)
    case params[:order]
    when 'author'
      { author: :asc }
    when 'publicationasc'
      { published_at: :asc }
    when 'publicationdesc'
      { published_at: :desc }
    when 'title'
      { name: :asc }
    else
      { score: :desc }
    end
  end

  def col_order_hash(params)
    case params[:order]
    when 'author'
      { owner: :asc }
    when 'publicationasc'
      { created_at: :asc }
    when 'publicationdesc'
      { created_at: :desc }
    when 'title'
      { name: :asc }
    else
      { name: :asc }
      # { score: :desc }
    end
  end

  def user_order_hash(params)
    { name: :asc }
  end

end
