namespace :create_relations do
  desc "Create relations"

  task :learning_object, [:log] => [:environment] do |t, args|
    desc "Create LearningObject relations based on its metadata"
    include Log::Logging

    args.with_defaults(:log => STDOUT)

    Log::Logging.logger = Log::DatabaseLogger.new(args.log)
    logger.level = Log::DatabaseLogger::CREATE

    new_topics = {}
    new_institutions = {}
    relations = []
    relations_order = []

    limit = 500
    offset = 0
    helper = LearningObjectRelations.new

    loop do
      logger.info " --> Creating LearningObjects relations from #{offset} to #{offset+limit}"

      begin
          learning_objects = LearningObject.limit(limit).offset(offset)

        rescue => e
          logger.warn "Database error, going to sleep"
          logger.error e
          sleep(10.seconds)
          next
        else
          break if learning_objects.empty?

          learning_objects.each do |learning_object|
              lo_topics = helper.parse_topics ( learning_object.get_metadata_values_of "dc.subject.category" )
              institution_name = learning_object.get_metadata_values_of("dc.creator").first

              lo_topics.each do |topic_name|
                unless Topic.exists?(name: topic_name)
                  new_topics[topic_name] ||= Topic.where(name: topic_name).new
                  new_topics[topic_name].learning_objects << learning_object
                  relations_order.push(topic_name)
                end
                end

              unless institution_name.nil? || Institution.exists?(name:institution_name)
                new_institutions[institution_name] ||= Institution.where(name: institution_name).new
              end

              relations_order.push nil

          end
        end
        offset += limit
      end

    Institution.import new_institutions.values
    logger.info " --> Institutions imported"
    Topic.import new_topics.values
    logger.info " --> Topics imported"

    relations_order.each_with_index do |relation, i|
      unless  relation.nil? || relations_order[i - 1].nil?
        parent_id =  Topic.find_by_name(relations_order[i - 1]).id
        child_id = Topic.find_by_name(relation).id
        unless TopicRelationship.exists?(parent_id: parent_id, child_id: child_id) || i == 0
          relations.push TopicRelationship.where(parent_id: parent_id, child_id: child_id ).new
        end
      end
    end

    TopicRelationship.import relations
    logger.info " --> TopicsRelations created"
    logger.close
  end


end
